@@ -1,17 +1,11 @@
-from typing import TypeVar
 from collections.abc import Callable
+from typing import TypeVar
+
+from theutilitybelt.functional.predicate import predicate
 
 TArg = TypeVar("TArg")
 
 
-def no_op(*args, **kwargs):
-    pass
-
-
-def identity(val: TArg) -> TArg:
-    return val
-
-
 def constant(val: TArg) -> Callable[..., TArg]:
     def fun(*_):
         return val
@@ -19,19 +13,6 @@ def constant(val: TArg) -> Callable[..., TArg]:
     return fun
 
 
-def compose(*funcs: Callable):
-    backwards = list(reversed(funcs))
-    initial, *rest = backwards
-
-    def composed_function(*args, **kwargs):
-        value = initial(*args, **kwargs)
-        for func in rest:
-            value = func(value)
-        return value
-
-    return composed_function
-
-
 def fn_not(func: Callable[..., bool]):
     def predicate(*args, **kwargs):
         return not func(*args, **kwargs)
@@ -64,3 +45,6 @@ def fn_xor(*funcs: Callable[..., bool]):
         return len([f for f in funcs if f(*args, **kwargs)]) % 2 == 1
 
     return predicate
+
+
+Predicate = predicate
